package user

import (
	"context"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"strings"
)
/*
func GetUserZoneName(id int64, dbname string) ([]string, error) {
	allZone, err := common.GetAllZoneCodeName(dbname)
	if err != nil {
		return nil, err
	}
	isSuper, zones, err := common.GetUserZone(id, dbname)
	if err != nil {
		return nil, err
	}
	ret := []string{}
	if isSuper {
		for _, v := range allZone {
			ret=append(ret, v)
		}
		return ret, nil
	}
	for k, _ := range zones {
		if v, ok := allZone[k]; ok {
			ret = append(ret, v)
		}
	}
	return ret, nil
}
*/
func getZonesByName(zoneName string, dbname string) (map[string]string,error) {
	p := model.NewZone(dbname)
	where := map[string]interface{}{
		"zone_name like":"%"+zoneName+"%",
	}
	zoneList := []model.Zone{}
	err := p.QueryAllNew(database.DB(), where, &zoneList)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		}
		return nil, errors.DataBaseError
	}
	codes := []string{}
	for _, v := range zoneList {
		codes = append(codes, v.ZoneCode)
	}
	list, err := p.GetZonesByCodes(database.DB(), codes)
	if err != nil {
		return nil, errors.DataBaseError
	}
	ret := map[string]string{}
	for _, v := range list {
		ret[v.ZoneCode] = ""
	}
	return ret, nil
}

func getUserIdsByZones(dbname string, zones []string) ([]int64,  error) {

	p := model.NewUserZone(dbname)

	where := map[string]interface{}{}

	if len(zones) > 0 {
		where["zone_code in"] = zones
	}
	list, err := p.ListAllGroup(database.DB(), where, "user_id")
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		}
		return nil,  errors.DataBaseError
	}

	ret := []int64{}
	for _, v := range list {
		ret = append(ret, v.UserId)
	}
	return ret,  nil
}


func UserList(ctx context.Context, req *pb_v1.UserListRequest)(reply *pb_v1.UserListReply, err error) {
	reply = &pb_v1.UserListReply{}
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.Uid == 0 || req.Organization == "" {
		return nil, errors.ParamsError
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	reply.Page = req.Page
	dbname := utils.GetDbName(req.Organization)


	// 获取登录用户的管理区域
	super, topLevel, zones,_, err := common.GetUserTopSubZone(req.Uid, dbname)
	if err != nil {
		return nil, err
	}
	loginUserInfo, err := common.GetUserBaseInfo(req.Uid, dbname)
	if err != nil {
		return nil, err
	}

	p := model.NewViewUser(dbname)
	mreq := &model.UserListReq{
		Uid:req.Uid,
		Limit:req.PageSize,
		Offset:(req.Page-1)*req.PageSize,
		Filter:req.Filter,
		ParentIds: fmt.Sprintf("%s%d,", loginUserInfo.Pids, req.Uid),
	}

	if !super && len(zones) == 0 {
		return reply, nil
	}


	loginUserSubZoneArray := []string{}
	filterZoneArray := []string{}



	if req.Zone == "" {
		for k, _ := range zones {
			loginUserSubZoneArray=append(loginUserSubZoneArray, k)
		}
	} else {
		// 获得请求过滤的区域包括子域
		filterZone, err := getZonesByName(req.Zone, dbname)
		if err != nil {
			return nil, err
		}
		if len(filterZone) == 0 {
			return reply, nil
		}

		for k, _ := range filterZone {
			filterZoneArray=append(filterZoneArray, k)
		}
	}

	if len(filterZoneArray) > 0 {
		userIds, err := getUserIdsByZones(dbname, filterZoneArray)
		if err != nil {
			return nil, err
		}
		if len(userIds) == 0 {
			return reply, nil
		}
		mreq.ZoneFilterUserIds = userIds
	}

	if len(loginUserSubZoneArray) > 0 {
		user := model.NewViewUser(dbname)
		userIds, err := user.GetUserIdBySubZones(database.DB(), topLevel, loginUserSubZoneArray)
		if err != nil {
			return nil, errors.DataBaseError
		}
		mreq.LoginUserSubZoneUserIds = userIds
		mreq.LoginUserSubZone = loginUserSubZoneArray
	}

	list, total, err := p.UserList(database.DB(), mreq)
	if err != nil {
		return nil, errors.DataBaseError
	}

	reply.Total = total
	if total == 0 {
		return reply, nil
	}

	reply.List = make([]*pb_v1.UserItem, len(list))
	for i, v := range list {
		reply.List[i] = &pb_v1.UserItem{
			Name:v.Name,
			Id:v.Id,
			Username:v.Username,
			Phone:v.Phone,
			Email:v.Email,
			Zones:strings.Split(v.Zones, ","),
		}
	}
	return reply, nil

}
