package zone

import (
	"context"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"github.com/jaryhe/gopkgs/cache"
)

func insertZone(db *gorm.DB, dbname string, req *pb_v1.ZoneAddRequest, lastCode string, level int) (string, error) {
	// 生成code并插入
	p := model.NewZone(dbname)
	subcode := ""
	if lastCode != "" {
		subcode = lastCode[len(req.ParentZoneCode):]
	}
	p.ZoneName = req.ZoneName
	p.ParentZoneCode = req.ParentZoneCode
	p.Level = level
	p.ZoneCode = req.ParentZoneCode+util.GenerateCode(subcode, "", 3)
	err := p.Insert(db)
	if err != nil {
		return "", errors.DataBaseError
	}
	return p.ZoneCode, nil
}

func insertUserZone(db *gorm.DB, dbname string, code string, req *pb_v1.ZoneAddRequest) (error) {
	p := model.NewUserZone(dbname)
	where := map[string]interface{}{
		"zone_code":req.ParentZoneCode,
	}
	list := []model.UserZone{}
	err := p.QueryAll(db, where, &list)
	if err != nil {
		// 没有用户使用父区域则关系表不用添加
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return errors.DataBaseError
	}
	for _, v := range list {
		t := model.NewUserZone(dbname)
		t.ZoneCode = code
		t.UserId = v.UserId
		err := t.Insert(db)
		if err != nil {
			return errors.DataBaseError
		}
		if err := common.DelUserZone(v.UserId, dbname); err != nil {
			return err
		}
	}

	return nil
}

func insertStaffZone(db *gorm.DB, dbname string, code string, req *pb_v1.ZoneAddRequest) (error) {
	//todo
	return nil
}

func insertDeviceZone(db *gorm.DB, dbname string, code string, req *pb_v1.ZoneAddRequest) (error) {
	//todo
	return nil
}



func CheckUserZonePermission(uid int64, dbname string, codes []string) error {
	super, userZone, err := common.GetUserZone(uid, dbname)
	if err != nil {
		return err
	}
	if !super {
		if len(userZone) == 0 {
			return errors.ZoneNotInRightZone
		}
		for _, code := range codes {
			if _, ok := userZone[code]; !ok {
				return errors.ZoneNotInRightZone
			}
		}

	}
	return nil
}

func ZoneAdd(ctx context.Context, req *pb_v1.ZoneAddRequest) (reply *pb_v1.ZoneAddReply, err error) {
	reply = &pb_v1.ZoneAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.OrganizationCode == "" || req.ZoneName == "" || req.ParentZoneCode == "" || req.Uid < 1 || req.ParentZoneCode == "000"{
		return nil, errors.ParamsError
	}


	dbname := utils.GetDbName(req.OrganizationCode)

	// 检查区域操作权限
	err = CheckUserZonePermission(req.Uid, dbname, []string{req.ParentZoneCode})
	if err != nil {
		return nil, err
	}
	// 检查名称
	p := model.NewZone(dbname)
	where := map[string]interface{}{
		"zone_name":req.ZoneName,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.Id > 0 {
		return nil, status.Error(10003, "区域名称已存在")
	}

	// 分布式锁
	dLock := &cache.Lock{Key: req.OrganizationCode+"zone_parent"+req.ParentZoneCode, Ttl: 60}
	if !dLock.RedisLock() {
		return nil, status.Error(10001, "系统繁忙")
	}
	defer dLock.RedisUnlock()

	// 检查父区域
	where = map[string]interface{}{
		"zone_code":req.ParentZoneCode,
	}
	err = p.FindLast(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, status.Error(10003, "父区域不存在")
		}
		return nil, errors.DataBaseError
	}
	newLevel := p.Level+1

	zone := model.NewZone(dbname)
	where = map[string]interface{}{
		"parent_zone_code":req.ParentZoneCode,
	}
	err = zone.FindLast(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}


	db := database.DB().Begin()
	reply.ZoneCode, err = insertZone(db, dbname, req, zone.ZoneCode, newLevel)
	if err != nil {
		db.Rollback()
		return nil, err
	}
	err = insertUserZone(db, dbname, reply.ZoneCode, req)
	if err != nil {
		db.Rollback()
		return nil, err
	}

	if err = common.DelAllZone(); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil

}
